/*
 *  Medard for Maemo.
 *  Copyright (C) 2011 Roman Moravcik
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef MEEGO_EDITION_HARMATTAN
#include <MLayout>
#include <MAction>
#include <MWidgetAction>
#include <MComboBox>
#endif

#include <QtGui>
#include <QSettings>

#include "aboutdialog.h"
#include "mainwindow.h"

#ifdef MEEGO_EDITION_HARMATTAN
MainWindow::MainWindow(QGraphicsItem *parent) : MApplicationPage(parent)
{
    m_downloader = new MedardDownloader();

    connect(m_downloader, SIGNAL(downloadFinished(const QString &, const QDateTime &)), this,
            SLOT(downloadedFinished(const QString &, const QDateTime &)));
    connect(m_downloader, SIGNAL(downloadFailed()), this, SLOT(downloadFailed()));

    m_forecast = new ForecastWidget();
    m_forecast->setPreferredSize(m_downloader->imageSize());

    m_forecastTypeLabel = new MLabel();
    m_forecastTypeLabel->setAlignment(Qt::AlignCenter);
    m_forecastTypeLabel->setPreferredSize(220, 80);

    m_forecastInitialDateLabel = new MLabel();
    m_forecastInitialDateLabel->setAlignment(Qt::AlignCenter);
    m_forecastInitialDateLabel->setWordWrap(true);
    m_forecastInitialDateLabel->setEnabled(false);

    m_forecastDateLabel = new MLabel();
    m_forecastDateLabel->setAlignment(Qt::AlignCenter);
    m_forecastDateLabel->setWordWrap(true);

    m_downloadRetryButton = new MButton(tr("Download again"));
    m_downloadRetryButton->setPreferredWidth(220);

    connect(m_downloadRetryButton, SIGNAL(clicked()), this, SLOT(downloadAgainClicked()));

    m_minusDayButton = new MButton(tr("-1 d"));
    m_minusDayButton->setPreferredWidth(50);
    m_plusDayButton = new MButton(tr("+1 d"));
    m_plusDayButton->setPreferredWidth(50);
    m_minusHourButton = new MButton(tr("-1 h"));
    m_minusHourButton->setPreferredWidth(50);
    m_plusHourButton = new MButton(tr("+1 h"));
    m_plusHourButton->setPreferredWidth(50);

    connect(m_minusDayButton, SIGNAL(clicked()), this, SLOT(minusDayClicked()));
    connect(m_plusDayButton, SIGNAL(clicked()), this, SLOT(plusDayClicked()));
    connect(m_minusHourButton, SIGNAL(clicked()), this, SLOT(minusHourClicked()));
    connect(m_plusHourButton, SIGNAL(clicked()), this, SLOT(plusHourClicked()));

    setupUi();
    setupMenu();

    loadSettings();
}
#else
MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
{
    m_downloader = new MedardDownloader();

    connect(m_downloader, SIGNAL(downloadFinished(const QString &, const QDateTime &)), this,
            SLOT(downloadedFinished(const QString &, const QDateTime &)));
    connect(m_downloader, SIGNAL(downloadFailed()), this, SLOT(downloadFailed()));

    m_forecast = new ForecastWidget();
    m_forecast->setFixedSize(m_downloader->imageSize());

    m_forecastTypeLabel = new QLabel();
    m_forecastTypeLabel->setAlignment(Qt::AlignCenter);
    m_forecastTypeLabel->setFixedSize(220, 80);

    m_forecastInitialDateLabel = new QLabel();
    m_forecastInitialDateLabel->setAlignment(Qt::AlignCenter);
    m_forecastInitialDateLabel->setDisabled(true);

    m_forecastDateLabel = new QLabel();
    m_forecastDateLabel->setAlignment(Qt::AlignCenter);

    m_downloadRetryButton = new QPushButton(tr("Download again"));

    connect(m_downloadRetryButton, SIGNAL(clicked()), this, SLOT(downloadAgainClicked()));

    m_minusDayButton = new QPushButton(tr("-1 d"));
    m_plusDayButton = new QPushButton(tr("+1 d"));
    m_minusHourButton = new QPushButton(tr("-1 h"));
    m_plusHourButton = new QPushButton(tr("+1 h"));

    connect(m_minusDayButton, SIGNAL(clicked()), this, SLOT(minusDayClicked()));
    connect(m_plusDayButton, SIGNAL(clicked()), this, SLOT(plusDayClicked()));
    connect(m_minusHourButton, SIGNAL(clicked()), this, SLOT(minusHourClicked()));
    connect(m_plusHourButton, SIGNAL(clicked()), this, SLOT(plusHourClicked()));

    setupUi();
    setupMenu();

    loadSettings();
}
#endif

MainWindow::~MainWindow()
{
    delete m_downloader;
}

#ifdef MEEGO_EDITION_HARMATTAN
void MainWindow::setupUi()
{
    setWindowTitle(tr("Medard"));
    setPannable(false);

    QGraphicsLinearLayout *mainLayout = new QGraphicsLinearLayout(Qt::Horizontal);
    centralWidget()->setLayout(mainLayout);

    mainLayout->addItem(m_forecast);

    QGraphicsLinearLayout *layout = new QGraphicsLinearLayout(Qt::Vertical);
    mainLayout->addItem(layout);

    layout->addItem(m_forecastTypeLabel);
    layout->addItem(m_forecastDateLabel);
    layout->addItem(m_forecastInitialDateLabel);
    layout->addItem(m_downloadRetryButton);

    QGraphicsLinearLayout *dayNavigationBox = new QGraphicsLinearLayout(Qt::Horizontal);
    dayNavigationBox->addItem(m_minusDayButton);
    dayNavigationBox->addItem(m_plusDayButton);
    layout->addItem(dayNavigationBox);

    QGraphicsLinearLayout *hourNavigationBox = new QGraphicsLinearLayout(Qt::Horizontal);
    hourNavigationBox->addItem(m_minusHourButton);
    hourNavigationBox->addItem(m_plusHourButton);
    layout->addItem(hourNavigationBox);

    hideNavigationButtons(false);
}
#else
void MainWindow::setupUi()
{
#ifdef Q_WS_MAEMO_5
    setAttribute(Qt::WA_Maemo5LandscapeOrientation, true);
#endif
    setWindowTitle(tr("Medard"));

    QWidget *widget = new QWidget;
    setCentralWidget(widget);

    QHBoxLayout *mainLayout = new QHBoxLayout();
    mainLayout->setMargin(8);
    mainLayout->setSpacing(4);
    widget->setLayout(mainLayout);

    mainLayout->addWidget(m_forecast);
    mainLayout->addSpacing(4);

    QVBoxLayout *layout = new QVBoxLayout();
    mainLayout->addLayout(layout);

    layout->addWidget(m_forecastTypeLabel);
    layout->addWidget(m_forecastDateLabel);
    layout->addSpacing(20);
    layout->addWidget(m_forecastInitialDateLabel);
    layout->addSpacing(20);
    layout->addWidget(m_downloadRetryButton);

    QHBoxLayout *dayNavigationBox = new QHBoxLayout();
    dayNavigationBox->addWidget(m_minusDayButton);
    dayNavigationBox->addWidget(m_plusDayButton);
    layout->addLayout(dayNavigationBox);

    QHBoxLayout *hourNavigationBox = new QHBoxLayout();
    hourNavigationBox->addWidget(m_minusHourButton);
    hourNavigationBox->addWidget(m_plusHourButton);
    layout->addLayout(hourNavigationBox);

    hideNavigationButtons(false);
}
#endif

#ifdef MEEGO_EDITION_HARMATTAN
void MainWindow::setupMenu()
{
    QStringList forecastDomainList;
    forecastDomainList << tr("Europe") 
                       << tr("Czech Republic");

    MWidgetAction *forecastDomainAction = new MWidgetAction(centralWidget());
    forecastDomainAction->setLocation(MAction::ApplicationMenuLocation);

    m_forecastDomainComboBox = new MComboBox;
    m_forecastDomainComboBox->setTitle(tr("Domain"));
    m_forecastDomainComboBox->setStyleName ("CommonComboBox");
    m_forecastDomainComboBox->setIconVisible(false);
    m_forecastDomainComboBox->addItems(forecastDomainList);
    forecastDomainAction->setWidget(m_forecastDomainComboBox);
    addAction(forecastDomainAction);
    connect(m_forecastDomainComboBox, SIGNAL(activated(int)), this, SLOT(forecastDomainChanged(int)));

    QStringList forecastTypeList;
    forecastTypeList << tr("Sea Level Pressure")
                     << tr("Precipitation")
                     << tr("Wind Velocity")
                     << tr("Cloudiness")
                     << tr("Temperature");

    MWidgetAction *forecastTypeAction = new MWidgetAction(centralWidget());
    forecastTypeAction->setLocation(MAction::ApplicationMenuLocation);

    m_forecastTypeComboBox = new MComboBox;
    m_forecastTypeComboBox->setTitle(tr("Forecast"));
    m_forecastTypeComboBox->setStyleName ("CommonComboBox");
    m_forecastTypeComboBox->setIconVisible(false);
    m_forecastTypeComboBox->addItems(forecastTypeList);
    forecastTypeAction->setWidget(m_forecastTypeComboBox);
    addAction(forecastTypeAction);
    connect(m_forecastTypeComboBox, SIGNAL(activated(int)), this, SLOT(forecastTypeChanged(int)));

    MAction *seaLevelPreasureAction = new MAction("icon-m-weather-cloudy", "", this);
    seaLevelPreasureAction->setLocation(MAction::ToolBarLocation);
    addAction(seaLevelPreasureAction);
    connect(seaLevelPreasureAction, SIGNAL(triggered()), this, SLOT(seaLevelPreasureMenuClicked()));

    MAction *precipitationAction = new MAction("icon-m-weather-rain", "", this);
    precipitationAction->setLocation(MAction::ToolBarLocation);
    addAction(precipitationAction);
    connect(precipitationAction, SIGNAL(triggered()), this, SLOT(precipitationMenuClicked()));

    MAction *cloudinessAction = new MAction("icon-m-weather-partly-sunny", "", this);
    cloudinessAction->setLocation(MAction::ToolBarLocation);
    addAction(cloudinessAction);
    connect(cloudinessAction, SIGNAL(triggered()), this, SLOT(cloudinessMenuClicked()));

    MAction *temperatureAction = new MAction("icon-m-weather-hot", "", this);
    temperatureAction->setLocation(MAction::ToolBarLocation);
    addAction(temperatureAction);
    connect(temperatureAction, SIGNAL(triggered()), this, SLOT(temperatureMenuClicked()));

    MAction *aboutAction = new MAction(this);
    aboutAction->setText(tr("About"));
    aboutAction->setLocation(MAction::ApplicationMenuLocation);
    addAction(aboutAction);
    connect(aboutAction, SIGNAL(triggered()), this, SLOT(aboutMenuClicked()));
}
#else
void MainWindow::setupMenu()
{
    QMenuBar *menu = new QMenuBar();
    setMenuBar(menu);

    m_domainActionGroup = new QActionGroup(this);
    m_domainActionGroup->setExclusive(true);

    QAction *domainAction;
    domainAction = new QAction(tr("Europe"), m_domainActionGroup);
    domainAction->setCheckable(true);
    domainAction = new QAction(tr("Czech Republic"), m_domainActionGroup);
    domainAction ->setCheckable(true);
    menu->addActions(m_domainActionGroup->actions());
    connect(m_domainActionGroup, SIGNAL(triggered(QAction *)), this, SLOT(forecastDomainChanged(QAction *)));

    QAction *seaLevelPreasureAction = new QAction(tr("Sea Level Pressure"), this);
    menu->addAction(seaLevelPreasureAction);
    connect(seaLevelPreasureAction, SIGNAL(triggered()), this, SLOT(seaLevelPreasureMenuClicked()));

    QAction *precipitationAction = new QAction(tr("Precipitation"), this);
    menu->addAction(precipitationAction);
    connect(precipitationAction, SIGNAL(triggered()), this, SLOT(precipitationMenuClicked()));

    QAction *windVelocityAction = new QAction(tr("Wind Velocity"), this);
    menu->addAction(windVelocityAction);
    connect(windVelocityAction, SIGNAL(triggered()), this, SLOT(windVelocityMenuClicked()));

    QAction *cloudinessAction = new QAction(tr("Cloudiness"), this);
    menu->addAction(cloudinessAction);
    connect(cloudinessAction, SIGNAL(triggered()), this, SLOT(cloudinessMenuClicked()));

    QAction *temperatureAction = new QAction(tr("Temperature"), this);
    menu->addAction(temperatureAction);
    connect(temperatureAction, SIGNAL(triggered()), this, SLOT(temperatureMenuClicked()));

    QAction *aboutAction = new QAction(tr("About"), this);
    menu->addAction(aboutAction);
    connect(aboutAction, SIGNAL(triggered()), this, SLOT(aboutMenuClicked()));
}
#endif

void MainWindow::loadSettings()
{
    QSettings settings;

    int forecastDomain = settings.value("ForecastDomain").toInt();
    int forecastType = settings.value("ForecastType").toInt();

    m_downloader->setForecastDomain((MedardDownloader::ForecastDomain) forecastDomain);
#ifdef MEEGO_EDITION_HARMATTAN
    m_forecastDomainComboBox->setCurrentIndex(forecastDomain);
#else
    m_domainActionGroup->actions().at(forecastDomain)->setChecked(true);
#endif

    switch ((MedardDownloader::ForecastType) forecastType) {
        case MedardDownloader::SeaLevelPressure:
            seaLevelPreasureMenuClicked();
            break;

        case MedardDownloader::Precipitation:
            precipitationMenuClicked();
            break;

        case MedardDownloader::WindVelocity:
            windVelocityMenuClicked();
            break;

        case MedardDownloader::Cloudiness:
            cloudinessMenuClicked();
            break;

        case MedardDownloader::Temperature:
            temperatureMenuClicked();
            break;
    }
}

void MainWindow::showNavigationButtons()
{
    m_downloadRetryButton->hide();

    m_minusDayButton->show();
    m_plusDayButton->show();
    m_minusHourButton->show();
    m_plusHourButton->show();
}

void MainWindow::hideNavigationButtons(bool showRetryButton)
{
    if (showRetryButton)
        m_downloadRetryButton->show();
    else
        m_downloadRetryButton->hide();

    m_minusDayButton->hide();
    m_plusDayButton->hide();
    m_minusHourButton->hide();
    m_plusHourButton->hide();
}

void MainWindow::updateNavigationButtons()
{
    if ((m_downloader->forecastDateOffset() - 24) < m_downloader->minForecastDateOffset()) {
#ifdef MEEGO_EDITION_HARMATTAN
        m_minusDayButton->setEnabled(false);
        m_plusDayButton->setEnabled(true);
#else
        m_minusDayButton->setDisabled(true);
        m_plusDayButton->setDisabled(false);
#endif
    } else if ((m_downloader->forecastDateOffset() + 24) > m_downloader->maxForecastDateOffset()) {
#ifdef MEEGO_EDITION_HARMATTAN
        m_minusDayButton->setEnabled(true);
        m_plusDayButton->setEnabled(false);
#else
        m_minusDayButton->setDisabled(false);
        m_plusDayButton->setDisabled(true);
#endif
    } else {
#ifdef MEEGO_EDITION_HARMATTAN
        m_minusDayButton->setEnabled(true);
        m_plusDayButton->setEnabled(true);
#else
        m_minusDayButton->setDisabled(false);
        m_plusDayButton->setDisabled(false);
#endif
    }

    if ((m_downloader->forecastDateOffset() - 1) < m_downloader->minForecastDateOffset()) {
#ifdef MEEGO_EDITION_HARMATTAN
        m_minusHourButton->setEnabled(false);
        m_plusHourButton->setEnabled(true);
#else
        m_minusHourButton->setDisabled(true);
        m_plusHourButton->setDisabled(false);
#endif
    } else if ((m_downloader->forecastDateOffset() + 1) > m_downloader->maxForecastDateOffset()) {
#ifdef MEEGO_EDITION_HARMATTAN
        m_minusHourButton->setEnabled(true);
        m_plusHourButton->setEnabled(false);
#else
        m_minusHourButton->setDisabled(false);
        m_plusHourButton->setDisabled(true);
#endif
    } else {
#ifdef MEEGO_EDITION_HARMATTAN
        m_minusHourButton->setEnabled(true);
        m_plusHourButton->setEnabled(true);
#else
        m_minusHourButton->setDisabled(false);
        m_plusHourButton->setDisabled(false);
#endif
    }
}

void MainWindow::setForecastType(const QString label, MedardDownloader::ForecastType type)
{
    m_forecastTypeLabel->setText(label);
    m_forecast->clearImage(false);
    m_downloader->setForecastType(type);
    m_downloader->downloadImage();

    QSettings settings;
    settings.setValue("ForecastType", type);
}

void MainWindow::setForecastDateOffset(int offset)
{
    m_forecast->clearImage(false);
    m_downloader->setForecastDateOffset(m_downloader->forecastDateOffset() + offset);
    m_downloader->downloadImage();
}

void MainWindow::seaLevelPreasureMenuClicked()
{
#ifdef MEEGO_EDITION_HARMATTAN
    m_forecastTypeComboBox->setCurrentIndex(MedardDownloader::SeaLevelPressure);
#endif
    setForecastType(tr("Sea Level Pressure"), MedardDownloader::SeaLevelPressure);
}

void MainWindow::precipitationMenuClicked()
{
#ifdef MEEGO_EDITION_HARMATTAN
    m_forecastTypeComboBox->setCurrentIndex(MedardDownloader::Precipitation);
#endif
    setForecastType(tr("Precipitation"), MedardDownloader::Precipitation);
}

void MainWindow::windVelocityMenuClicked()
{
#ifdef MEEGO_EDITION_HARMATTAN
    m_forecastTypeComboBox->setCurrentIndex(MedardDownloader::WindVelocity);
#endif
    setForecastType(tr("Wind Velocity"), MedardDownloader::WindVelocity);
}

void MainWindow::cloudinessMenuClicked()
{
#ifdef MEEGO_EDITION_HARMATTAN
    m_forecastTypeComboBox->setCurrentIndex(MedardDownloader::Cloudiness);
#endif
    setForecastType(tr("Cloudiness"), MedardDownloader::Cloudiness);
}

void MainWindow::temperatureMenuClicked()
{
#ifdef MEEGO_EDITION_HARMATTAN
    m_forecastTypeComboBox->setCurrentIndex(MedardDownloader::Temperature);
#endif
    setForecastType(tr("Temperature"), MedardDownloader::Temperature);
}

void MainWindow::aboutMenuClicked()
{
    AboutDialog *dialog = new AboutDialog();
#ifdef MEEGO_EDITION_HARMATTAN
    dialog->appear(MSceneWindow::DestroyWhenDismissed);
#else
    dialog->exec();
#endif
}

void MainWindow::downloadAgainClicked()
{
    m_forecast->clearImage(false);
    m_downloader->downloadImage();

    hideNavigationButtons(false);
}

void MainWindow::plusDayClicked()
{
    setForecastDateOffset(+24);
}

void MainWindow::minusDayClicked()
{
    setForecastDateOffset(-24);
}

void MainWindow::plusHourClicked()
{
    setForecastDateOffset(+1);
}

void MainWindow::minusHourClicked()
{
    setForecastDateOffset(-1);
}

#ifdef MEEGO_EDITION_HARMATTAN
void MainWindow::forecastDomainChanged(int index)
{
    m_forecast->clearImage(false);

    if (index == 0)
        m_downloader->setForecastDomain(MedardDownloader::Europe);
    else
        m_downloader->setForecastDomain(MedardDownloader::CzechRepublic);

    m_downloader->downloadImage();

    QSettings settings;
    settings.setValue("ForecastDomain", index);
}

void MainWindow::forecastTypeChanged(int index)
{
    switch ((MedardDownloader::ForecastType) index) {
        case MedardDownloader::SeaLevelPressure:
            setForecastType(tr("Sea Level Pressure"), MedardDownloader::SeaLevelPressure);
            break;

        case MedardDownloader::Precipitation:
            setForecastType(tr("Precipitation"), MedardDownloader::Precipitation);
            break;

        case MedardDownloader::WindVelocity:
            setForecastType(tr("Wind Velocity"), MedardDownloader::WindVelocity);
            break;

        case MedardDownloader::Cloudiness:
            setForecastType(tr("Cloudiness"), MedardDownloader::Cloudiness);
            break;

        case MedardDownloader::Temperature:
            setForecastType(tr("Temperature"), MedardDownloader::Temperature);
            break;
    }
}

#else
void MainWindow::forecastDomainChanged(QAction *action)
{
    int forecastDomain = m_domainActionGroup->actions().indexOf(action);

    m_forecast->clearImage(false);

    if (forecastDomain == 0)
        m_downloader->setForecastDomain(MedardDownloader::Europe);
    else
        m_downloader->setForecastDomain(MedardDownloader::CzechRepublic);

    m_downloader->downloadImage();

    QSettings settings;
    settings.setValue("ForecastDomain", forecastDomain);
}
#endif

void MainWindow::downloadedFinished(const QString &filename, const QDateTime &date)
{
    m_forecast->setImage(filename);
    m_forecastInitialDateLabel->setText(tr("Forecast from:\n") +
                                          m_downloader->forecastInitialDate().toString("dd.MM.yyyy, hh:mm"));

    /* upcase the first letter of name of day */
    QString temp = date.toString("dddd\ndd.MM.yyyy, hh:mm");
    m_forecastDateLabel->setText(temp.replace(0, 1, temp.at(0).toUpper()));

    showNavigationButtons();
    updateNavigationButtons();
}

void MainWindow::downloadFailed()
{
    m_forecast->clearImage(true);
    m_forecastInitialDateLabel->setText("");
    m_forecastDateLabel->setText("");

    hideNavigationButtons(true);
}
